@@ -36,6 +36,7 @@ class Gemini:
     def __init__(
         self,
         session: Optional[requests.Session] = None,
+        nonce: str = None,
         cookies: Optional[Dict[str, str]] = None,
         cookie_fp: str = None,
         auto_cookies: bool = False,
@@ -52,6 +53,7 @@ class Gemini:
         self.timeout = timeout
         self.session = session or self._initialize_session(cookies, cookie_fp)
         self.base_url: str = HOST
+        self.nonce = None
 
     def _initialize_session(
         self, cookies: Optional[Dict[str, str]], cookie_fp: Optional[str]
@@ -184,7 +186,10 @@ class Gemini:
             ) from e
 
         sid: str = self._search_regex(response.text, r'"FdrFJe":"([\d-]+)"', "SID")
-        nonce: str = self._search_regex(response.text, r'"SNlM0e":"(.*?)"', "nonce")
+        try:
+            nonce = self._search_regex(response.text, r'"SNlM0e":"(.*?)"', "nonce")
+        except:
+            nonce = self.nonce
 
         return sid, nonce
 
@@ -261,6 +266,16 @@ class Gemini:
         """Sends a request and returns the response text and status code."""
         try:
             sid, nonce = self._get_sid_and_nonce()
+            if nonce is None and self.nonce is None:
+                raise ValueError(
+                    "Error: Cannot find nonce value.\n"
+                    "Please refresh the Gemini web page, send any prompt, re-export the cookie, "
+                    "and manually collect the nonce value.\n"
+                    "Refer to the following page: https://github.com/dsdanielpark/Gemini-API?tab=readme-ov-file#authentication"
+                )
+            else:
+                nonce = self.nonce
+
             params = self._construct_params(sid)
             data = self._construct_payload(prompt, nonce)
             response = self.session.post(
@@ -273,9 +288,9 @@ class Gemini:
             response.raise_for_status()
             return response.text, response.status_code
         except ConnectionError as e:
-            raise ConnectionError(f"Connection failed: {e}")
+            raise ConnectionError(f"Connection failed: {e}") from e
         except RequestException as e:
-            raise RequestException(f"Request failed: {e}")
+            raise RequestException(f"Request failed: {e}") from e
 
     def generate_content(self, prompt: str) -> str:
         """Generates content based on the prompt, raising an exception for non-200 responses."""
